@@ -379,6 +379,7 @@ namespace Sass {
     // solve the double dispatch problem by using RTTI information via dynamic cast
     if (const Pseudo_Selector* lhs = Cast<Pseudo_Selector>(this)) {return *lhs == rhs; }
     else if (const Wrapped_Selector* lhs = Cast<Wrapped_Selector>(this)) {return *lhs == rhs; }
+    else if (const Element_Selector* lhs = Cast<Element_Selector>(this)) {return *lhs == rhs; }
     else if (const Attribute_Selector* lhs = Cast<Attribute_Selector>(this)) {return *lhs == rhs; }
     else if (name_ == rhs.name_)
     { return is_ns_eq(rhs); }
@@ -390,6 +391,7 @@ namespace Sass {
     // solve the double dispatch problem by using RTTI information via dynamic cast
     if (const Pseudo_Selector* lhs = Cast<Pseudo_Selector>(this)) {return *lhs < rhs; }
     else if (const Wrapped_Selector* lhs = Cast<Wrapped_Selector>(this)) {return *lhs < rhs; }
+    else if (const Element_Selector* lhs = Cast<Element_Selector>(this)) {return *lhs < rhs; }
     else if (const Attribute_Selector* lhs = Cast<Attribute_Selector>(this)) {return *lhs < rhs; }
     if (is_ns_eq(rhs))
     { return name_ < rhs.name_; }
@@ -661,12 +663,48 @@ namespace Sass {
   {
     if (Attribute_Selector_Ptr_Const w = Cast<Attribute_Selector>(&rhs))
     {
-      return *this == *w;
+      return is_ns_eq(rhs) &&
+             name() == rhs.name() &&
+             *this == *w;
     }
+    return false;
+  }
+
+  bool Element_Selector::operator< (const Element_Selector& rhs) const
+  {
+    if (is_ns_eq(rhs))
+    { return name() < rhs.name(); }
+    return ns() < rhs.ns();
+  }
+
+  bool Element_Selector::operator< (const Simple_Selector& rhs) const
+  {
+    if (Element_Selector_Ptr_Const w = Cast<Element_Selector>(&rhs))
+    {
+      return *this < *w;
+    }
+    if (is_ns_eq(rhs))
+    { return name() < rhs.name(); }
+    return ns() < rhs.ns();
+  }
+
+  bool Element_Selector::operator== (const Element_Selector& rhs) const
+  {
     return is_ns_eq(rhs) &&
            name() == rhs.name();
   }
 
+  bool Element_Selector::operator== (const Simple_Selector& rhs) const
+  {
+    if (Element_Selector_Ptr_Const w = Cast<Element_Selector>(&rhs))
+    {
+      return is_ns_eq(rhs) &&
+             name() == rhs.name() &&
+             *this == *w;
+    }
+    return false;
+  }
+
   bool Pseudo_Selector::operator== (const Pseudo_Selector& rhs) const
   {
     if (is_ns_eq(rhs) && name() == rhs.name())
