@@ -20,6 +20,7 @@
 #include "context.hpp"
 #include "prelexer.hpp"
 #include "utf8_string.hpp"
+#include "sass_functions.hpp"
 #include "sass2scss.h"
 
 #ifdef _WIN32
@@ -99,7 +100,7 @@ namespace Sass {
     // helper function to find the last directory seperator
     inline size_t find_last_folder_separator(const std::string& path, size_t limit = std::string::npos)
     {
-      size_t pos = std::string::npos;
+      size_t pos;
       size_t pos_p = path.find_last_of('/', limit);
       #ifdef _WIN32
         size_t pos_w = path.find_last_of('\\', limit);
@@ -150,7 +151,7 @@ namespace Sass {
       pos = 0; // remove all self references inside the path string
       while((pos = path.find("/./", pos)) != std::string::npos) path.erase(pos, 2);
 
-      pos = 0; // remove all leading and trailing self references
+      // remove all leading and trailing self references
       while(path.length() > 1 && path.substr(0, 2) == "./") path.erase(0, 2);
       while((pos = path.length()) > 1 && path.substr(pos - 2) == "/.") path.erase(pos - 2);
 
@@ -301,13 +302,9 @@ namespace Sass {
     // (2) underscore + given
     // (3) underscore + given + extension
     // (4) given + extension
-    std::vector<Include> resolve_includes(const std::string& root, const std::string& file)
+    std::vector<Include> resolve_includes(const std::string& root, const std::string& file, const std::vector<std::string>& exts)
     {
       std::string filename = join_paths(root, file);
-      // supported extensions
-      const std::vector<std::string> exts = {
-        ".scss", ".sass", ".css"
-      };
       // split the filename
       std::string base(dir_name(file));
       std::string name(base_name(file));
@@ -321,13 +318,13 @@ namespace Sass {
       abs_path = join_paths(root, rel_path);
       if (file_exists(abs_path)) includes.push_back({{ rel_path, root }, abs_path });
       // next test exts plus underscore
-      for(auto ext : exts) {
+      for (auto __ext = (exts).begin(); __ext != (exts).end(); ++__ext) { auto ext = *(__ext);
         rel_path = join_paths(base, "_" + name + ext);
         abs_path = join_paths(root, rel_path);
         if (file_exists(abs_path)) includes.push_back({{ rel_path, root }, abs_path });
       }
       // next test plain name with exts
-      for(auto ext : exts) {
+      for (auto __ext = (exts).begin(); __ext != (exts).end(); ++__ext) { auto ext = *(__ext);
         rel_path = join_paths(base, name + ext);
         abs_path = join_paths(root, rel_path);
         if (file_exists(abs_path)) includes.push_back({{ rel_path, root }, abs_path });
@@ -336,8 +333,41 @@ namespace Sass {
       return includes;
     }
 
-    // helper function to resolve a filename
+    std::vector<std::string> find_files(const std::string& file, const std::vector<std::string> paths)
+    {
+      std::vector<std::string> includes;
+      for (auto __path = (paths).begin(); __path != (paths).end(); ++__path) { std::string path = *(__path);
+        std::string abs_path(join_paths(path, file));
+        if (file_exists(abs_path)) includes.push_back(abs_path);
+      }
+      return includes;
+    }
+
+    std::vector<std::string> find_files(const std::string& file, struct Sass_Compiler* compiler)
+    {
+      // get the last import entry to get current base directory
+      // struct Sass_Options* options = sass_compiler_get_options(compiler);
+      Sass_Import_Entry import = sass_compiler_get_last_import(compiler);
+      const std::vector<std::string>& incs = compiler->cpp_ctx->include_paths;
+      // create the vector with paths to lookup
+      std::vector<std::string> paths(1 + incs.size());
+      paths.push_back(dir_name(import->abs_path));
+      paths.insert(paths.end(), incs.begin(), incs.end());
+      // dispatch to find files in paths
+      return find_files(file, paths);
+    }
+
+    // helper function to search one file in all include paths
+    // this is normally not used internally by libsass (C-API sugar)
     std::string find_file(const std::string& file, const std::vector<std::string> paths)
+    {
+      if (file.empty()) return file;
+      auto res = find_files(file, paths);
+      return res.empty() ? "" : res.front();
+    }
+
+    // helper function to resolve a filename
+    std::string find_include(const std::string& file, const std::vector<std::string> paths)
     {
       // search in every include path for a match
       for (size_t i = 0, S = paths.size(); i < S; ++i)
@@ -349,20 +379,6 @@ namespace Sass {
       return std::string("");
     }
 
-    // inc paths can be directly passed from C code
-    std::string find_file(const std::string& file, const char* paths[])
-    {
-      if (paths == 0) return std::string("");
-      std::vector<std::string> includes(0);
-      // includes.push_back(".");
-      const char** it = paths;
-      while (it && *it) {
-        includes.push_back(*it);
-        ++it;
-      }
-      return find_file(file, includes);
-    }
-
     // try to load the given filename
     // returned memory must be freed
     // will auto convert .sass files
@@ -414,5 +430,25 @@ namespace Sass {
       }
     }
 
+    // split a path string delimited by semicolons or colons (OS dependent)
+    std::vector<std::string> split_path_list(const char* str)
+    {
+      std::vector<std::string> paths;
+      if (str == NULL) return paths;
+      // find delimiter via prelexer (return zero at end)
+      const char* end = Prelexer::find_first<PATH_SEP>(str);
+      // search until null delimiter
+      while (end) {
+        // add path from current position to delimiter
+        paths.push_back(std::string(str, end - str));
+        str = end + 1; // skip delimiter
+        end = Prelexer::find_first<PATH_SEP>(str);
+      }
+      // add path from current position to end
+      paths.push_back(std::string(str));
+      // return back
+      return paths;
+    }
+
   }
 }
