@@ -5,7 +5,7 @@ module SassSpec::CLI
 
   def self.parse
     options = {
-      engine_adapter: SassEngineAdapter.new("sass"),
+      engine_adapter: SassEngineAdapter.new,
       spec_directory: nil,
       generate: false,
       tap: false,
@@ -56,7 +56,11 @@ Make sure the command you provide prints to stdout.
       end
 
       opts.on("-c", "--command COMMAND", "Sets a specific binary to run (defaults to '#{options[:engine_adapter]}')") do |v|
-        options[:engine_adapter] = ExecutableEngineAdapater.new(v)
+        options[:engine_adapter] = ExecutableEngineAdapter.new(v)
+      end
+
+      opts.on("--dart PATH", "Run Dart Sass, whose repo should be at the given path.") do |path|
+        options[:engine_adapter] = DartEngineAdapter.new(path)
       end
 
       opts.on("-g", "--generate", "Run test(s) and generate expected output file(s).") do
@@ -97,8 +101,12 @@ Make sure the command you provide prints to stdout.
         options[:spec_directory] = File.expand_path(spec_dir)
       end
 
-      opts.on("--migrate", "Copy tests that fail and make them pass for the current version.") do
-        options[:migrate] = true
+      opts.on("--migrate-version", "Copy tests that fail and make them pass for the current version.") do
+        options[:migrate_version] = true
+      end
+
+      opts.on("--migrate-impl", "Copy tests that fail and make them pass for the current implementatino.") do
+        options[:migrate_impl] = true
       end
 
       opts.on("--silent", "Don't show any logs") do
