@@ -44,13 +44,19 @@ class SassSpec::Runner
 
     result = Minitest.run(minioptions)
 
-    if @options[:probe_todo]
-      puts "Reporting successfully probed todo tests"
+    if @options[:run_todo]
+      passing = []
       test_cases.each do |test_case|
         if test_case.todo? && test_case.result?
-          puts test_case.folder
+          passing << test_case.folder
         end
       end
+      if passing.any?
+        puts "The following tests pass but were marked as TODO for #{@options[:engine_adapter].describe}:"
+        puts passing.join("\n")
+      else
+        puts "Note: All tests marked as TODO for #{@options[:engine_adapter].describe} are still failing."
+      end
     end
 
     result
@@ -71,19 +77,29 @@ class SassSpec::Runner
     @language_version
   end
 
+  def impl
+    @options[:engine_adapter].describe
+  end
 
   def _get_cases
     cases = []
     get_input_files().each do |filename|
       folder = File.dirname(filename)
       metadata = SassSpec::TestCaseMetadata.new(folder)
-      unless metadata.valid_for_version(language_version)
+      unless metadata.valid_for_version?(language_version)
         if @options[:verbose]
           warn "#{metadata.name} does not apply to Sass #{language_version}"
         end
         next
       end
 
+      unless metadata.valid_for_impl?(impl)
+        if @options[:verbose]
+          warn "#{metadata.name} does not apply to #{impl}"
+        end
+        next
+      end
+
       next unless filename.include?(@options[:filter] || "")
 
       if  @options[:only_output_styles] && @options[:only_output_styles].any?
